unit DXRectTracker;

{*******************************************************************************
 Project:	RectTracker
 Description:	CRectTracker for Delphi
 Date:	2017.12.27
 Target:	Win32, Delphi 7
 Author: Dai Xu, daixu61@hotmail.com
 QQ: 82429536
 Blog: http://www.cnblogs.com/popapa/p/DXRectTracker.html
 See: http://www.2ccc.com/article.asp?articleid=4138
*******************************************************************************}

interface

uses
  Types, Messages,SysUtils, Classes, Graphics, Controls, Forms;

type
  TMousePosType = (mpLeft,mpRight,mpTop,mpBottom,mpLeftTop,mpRightTop,mpLeftBottom,mpRightBottom,mpInBox,mpOutBox);

  TDXRectTracker = class(TGraphicControl)
  private
    FDragging: boolean;
    FHandleSize: integer;
    FBorderWidth: integer;
    FMinSize: integer;
    FTrackerType: TMousePosType;
    FX,FY: integer;
    procedure WMMouseMove(var msg: TWMMouseMove);message WM_MOUSEMOVE;
    procedure WMLButtonDown(var msg: TWMLButtonDown);message WM_LBUTTONDOWN;
    procedure WMLButtonUp(var msg: TWMLButtonUp);message WM_LBUTTONUP;
    function GetMousePos(X, Y: Integer): TMousePosType;
    { Private declarations }
  protected
    procedure Paint; override;
    { Protected declarations }
  public
    { Public declarations }
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  published
    { Published declarations }
    property OnResize;
    property Visible;
    property Left;
    property Top;
    property Width;
    property Height;
  end;

  procedure Register;

implementation
{$R *.dcr}

const
  DefaultSize=65;
  DefaultHandleSize=5;
  DefaultBorderWidth=1;
  
procedure Register;
begin
  RegisterComponents('Samples', [TDXRectTracker]);
end;

constructor TDXRectTracker.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  
  if Screen.Width<=1024 then
  begin
    FHandleSize:= DefaultHandleSize;
//    FBorderWidth:= DefaultBorderWidth;
  end
  else if Screen.Width<=1920 then
  begin
    FHandleSize:= DefaultHandleSize + 2;
//    FBorderWidth:= DefaultBorderWidth + 1;
  end
  else
  begin
    FHandleSize:= DefaultHandleSize + 4;
//    FBorderWidth:= DefaultBorderWidth + 2;
  end;

  Fx:= -1;
  Fy:= -1;
  FMinSize:=  FHandleSize * 3;
  FTrackerType:= mpOutBox;
  FDragging:= false;
  
  Width:= DefaultSize;
  Height:= DefaultSize;
  Color:= clBlack;
end;

destructor TDXRectTracker.Destroy;
begin
  inherited;
end;

procedure TDXRectTracker.Paint;
var
  padding:integer;
  middleX, rightX, middleY, bottomY:integer;
begin
  inherited;
  //draw dotted border
  Canvas.Brush.Style:= bsClear;
  Canvas.Pen.Style:= psDot;
  Canvas.Pen.Width:= FBorderWidth;
  Canvas.Pen.Mode:= pmNotXor;
  padding:= FHandleSize div 2;
  Canvas.Rectangle(padding , padding, Width - padding, Height-padding);

  //draw dragging handle
  rightX:= width - FHandleSize;
  middleX:= rightX div 2;
  bottomY:= height - FHandleSize;
  middleY:= bottomY div 2;
  Canvas.Brush.Style:= bsSolid;
  Canvas.Brush.Color:= Color;
  Canvas.Pen.Style:= psSolid;
  Canvas.Pen.Mode:= pmCopy;
  Canvas.Rectangle(0, 0, FHandleSize, FHandleSize);
  Canvas.Rectangle(middleX, 0, middleX + FHandleSize, FHandleSize);
  Canvas.Rectangle(rightX, 0, Width, FHandleSize);
  Canvas.Rectangle(rightX, middleY, Width, middleY + FHandleSize);
  Canvas.Rectangle(rightX, bottomY, Width, Height);
  Canvas.Rectangle(middleX, bottomY, middleX + FHandleSize, Height);
  Canvas.Rectangle(0, bottomY, FHandleSize, Height);
  Canvas.Rectangle(0, middleY, FHandleSize, middleY + FHandleSize);
end;

procedure TDXRectTracker.WMMouseMove(var msg: TWMMouseMove);
var
  dx,dy: integer;
  x1, y1, x2, y2: integer;
label
  DoDefault;
begin
  if not FDragging then
    goto DoDefault;
  dx:= msg.XPos - Fx;
  dy:= msg.YPos - Fy;
  if (dx=0) and (dy=0) then
    goto DoDefault;
  x1:= Left;
  y1:= Top;
  x2:= Left + Width;
  y2:= Top + Height;

  case FTrackerType of
    mpLeft:
      begin
        inc(x1, dx);
      end;
    mpRight:
      begin
        inc(x2, dx);
        Fx:= msg.XPos;
      end;
    mpTop:
      begin
        inc(y1, dy);
      end;
    mpBottom:
      begin
        inc(y2, dy);
        Fy:= msg.YPos;
      end;
    mpLeftTop:
      begin
        inc(x1, dx);
        inc(y1, dy);
      end;
    mpRightBottom:
      begin
        inc(x2, dx);
        inc(y2, dy);
        Fx:= msg.XPos;
        Fy:= msg.YPos;
      end;
    mpLeftBottom:
      begin
        inc(x1, dx);
        inc(y2, dy);
        Fy:= msg.YPos;
      end;
    mpRightTop:
      begin
        inc(x2, dx);
        inc(y1, dy);
        Fx:= msg.XPos;
      end;
    mpInBox:
      begin
        inc(x1, dx);
        inc(y1, dy);
        inc(x2, dx);
        inc(y2, dy);
      end;
  end;

  // detect boundary of parent only when dragging
  if x1<0 then
  begin
    x1:= 0;
    if FTrackerType=mpInBox then
      x2:= Width;
  end
  else if x2>Parent.ClientWidth then
  begin
    x2:= Parent.ClientWidth;
    if FTrackerType=mpInBox then
      x1:= Parent.ClientWidth - Width;
  end;
  if y1<0 then
  begin
    y1:= 0;
    if FTrackerType=mpInBox then
      y2:= Height;
  end
  else if y2>Parent.ClientHeight then
  begin
    y2:= Parent.ClientHeight;
    if FTrackerType=mpInBox then
      y1:= Parent.ClientHeight - Height;
  end;

  if (x2-x1>=FMinSize) and (y2-y1>=FMinSize) then
    SetBounds(x1, y1, x2-x1, y2-y1);

DoDefault:
  case GetMousePos(msg.XPos, msg.YPos) of
    mpLeftTop, mpRightBottom:
      Cursor:= crSizeNWSE;
    mpLeftBottom, mpRightTop:
      Cursor:= crSizeNESW;
    mpLeft, mpRight:
      Cursor:= crSizeWE;
    mpTop, mpBottom:
      Cursor:= crSizeNS;
    mpInBox:
      Cursor:= crSizeAll;
    mpOutBox:
      Cursor:= crDefault;
  end;
  inherited;
end;

procedure TDXRectTracker.WMLButtonDown(var msg: TWMLButtonDown);
begin
  FDragging:= true;
  Fx:= msg.XPos;
  Fy:= msg.YPos;
  FTrackerType:= GetMousePos(Fx, Fy);
  inherited;
end;

procedure TDXRectTracker.WMLButtonUp(var msg: TWMLButtonUp);
begin
  FDragging:= false;
  Fx:= -1;
  Fy:= -1;
  FTrackerType:= mpOutBox;
  inherited;
end;

function TDXRectTracker.GetMousePos(X, Y: Integer): TMousePosType;
var
  middleX, rightX, middleY, bottomY: integer;
  pt: TPoint;
begin
  Result:= mpOutBox;
  rightX:= Width - FHandleSize;
  middleX:= rightX div 2;
  bottomY:= Height - FHandleSize;
  middleY:= bottomY div 2;
  pt.X:= X;
  pt.Y:= Y;
  
  if ptInRect(Rect(0, 0, FHandleSize, FHandleSize), pt) then
    Result:= mpLeftTop
  else if ptInRect(Rect(middleX, 0, middleX + FHandleSize, FHandleSize), pt) then
    Result:= mpTop
  else if ptInRect(Rect(rightX, 0, Width, FHandleSize), pt) then
    Result:= mpRightTop
  else if ptInRect(Rect(rightX, middleY, Width, middleY + FHandleSize), pt) then
    Result:= mpRight    
  else if ptInRect(Rect(rightX, bottomY, Width, Height), pt) then
    Result:= mpRightBottom
  else if ptInRect(Rect(middleX, bottomY, middleX + FHandleSize, Height), pt) then  //
    Result:= mpBottom
  else if ptInRect(Rect(0, bottomY, FHandleSize, Height), pt) then
    Result:= mpLeftBottom
  else if ptInRect(Rect(0, middleY, FHandleSize, middleY + FHandleSize), pt) then  //
    Result:= mpLeft
  else if ptInRect(GetClientRect, pt) then
    Result:= mpInBox;
end;

end.

