unit FFMPEGindex;

interface

uses
  SysUtils, Classes, Math, Contnrs;
  
Type

  PFrameInfo = ^TFrameInfo;
  TFrameInfo = packed record
    PTS, Pos : int64;
    FrameType: integer;
    KeyFrame : boolean;
  end;
  
  TFrameIndexer = class
  private
    FFrameList: TList;
    
    function GetCount:integer;

  public
    constructor Create;
    destructor Destroy; override;
    function ToString:string;

    procedure Clear;
    procedure Sort;
    function IndexOf(pts:int64):integer;
    function Add(pts, pos:int64; frameType:integer; isKeyFrame: boolean):integer;
    function GetPos(index:integer):int64;
    function GetPts(index:integer):int64;
    function GetPrevKeyFrameIndex(index:integer):integer;
    property FrameCount: integer read GetCount;
  end;

implementation

constructor TFrameIndexer.Create;
begin
  inherited Create;
  FFrameList:= TList.Create;
end;

destructor TFrameIndexer.Destroy;
begin
  Clear;
  FFrameList.Free;
  inherited Destroy;
end;

function TFrameIndexer.ToString:string;
var
  i:integer;
  str:TStringList;
  p:PFrameInfo;
begin
  str:=TStringList.Create;
  for i:=0 to FFrameList.Count-1 do
  begin
    p:= FFrameList[i];
    str.Add(format('%d, pts:%d, pos:%d, type:%d, key:%d', [i, p^.pts, p^.pos, p^.FrameType, p^.KeyFrame]));
  end;
  result:=str.Text;
  str.free;
end;

procedure TFrameIndexer.Clear;
var
  i:integer;
begin
  for i:=FFrameList.Count-1 downto 0 do
  begin
    dispose(FFrameList[i]);
    FFrameList[i]:=nil;
  end;
  FFrameList.Clear;
end;

procedure TFrameIndexer.Sort;
function DoCompare(Item1, Item2: Pointer): Integer;
var
  pts1,pts2:int64;
begin
  pts1:= TFrameInfo(item1^).pts;
  pts2:= TFrameInfo(item2^).pts;
  Result := int64(CompareValue(pts1, pts2));
{
  if pts1<pts2 then
    result:=1
  else if pts1>pts2 then
    result:=-1
  else
    result:=0;
}
end;
begin
  FFrameList.Sort(@DoCompare);
end;

function TFrameIndexer.Add(pts, pos:int64; frameType:integer; isKeyFrame: boolean):integer;
var
  p:PFrameInfo;
begin
  new(p);
  p^.PTS:= pts;
  p^.Pos:= pos;
  p^.FrameType:= frameType;
  p^.KeyFrame:= isKeyFrame;
  result:= FFrameList.Add(p);
end;

function TFrameIndexer.GetCount:integer;
begin
  result:= FFrameList.Count;
end;

function TFrameIndexer.GetPos(index:integer):int64;
begin
  result:= TFrameInfo(FFrameList[index]^).Pos;
end;

function TFrameIndexer.GetPts(index:integer):int64;
begin
  result:= TFrameInfo(FFrameList[index]^).PTS;
end;

function TFrameIndexer.IndexOf(pts:int64):integer;
var
  i:integer;
begin
  result:=-1;
  if FFrameList.count<=0 then
    exit;
  if (pts<TFrameInfo(FFrameList[0]^).pts) then
  begin
    result:=0;
    exit;
  end
  else if (pts>TFrameInfo(FFrameList[FFrameList.count-1]^).pts) then
  begin
    result:=FFrameList.count-1;
    exit;
  end;
  for i:=0 to FFrameList.Count-1 do
  begin
    if TFrameInfo(FFrameList[i]^).pts>=pts then
    begin
      result:=i;
      exit;
    end;
  end;
end;

function TFrameIndexer.GetPrevKeyFrameIndex(index:integer):integer;
var
  i:integer;
begin
  result:=0;
  if index<=1 then
  begin
    result:=0;
    exit;
  end;
  for i:=index-1 downto 0 do
  begin
    if TFrameInfo(FFrameList[i]^).keyframe then
    begin
      result:=i;
      exit;
    end;
  end;
end;

end.
