unit LibFFMS2;

{
  FFMS_GetNumTracks will modify errorinfo memory!
}

interface

const
	FFMS_SEEK_LINEAR_NO_RW	= -1;
	FFMS_SEEK_LINEAR		= 0;
	FFMS_SEEK_NORMAL		= 1;
	FFMS_SEEK_UNSAFE		= 2;
	FFMS_SEEK_AGGRESSIVE	= 3;

  	// No error
	FFMS_ERROR_SUCCESS = 0;

	// Main types - where the error occurred
	FFMS_ERROR_INDEX = 1;			// index file handling
	FFMS_ERROR_INDEXING =2;			// indexing
	FFMS_ERROR_POSTPROCESSING = 3;		// video postprocessing (libpostproc)
	FFMS_ERROR_SCALING=4;				// image scaling (libswscale)
	FFMS_ERROR_DECODING=5;			// audio/video decoding
	FFMS_ERROR_SEEKING=6;				// seeking
	FFMS_ERROR_PARSER=7;				// file parsing
	FFMS_ERROR_TRACK=8;				// track handling
	FFMS_ERROR_WAVE_WRITER=9;			// WAVE64 file writer
	FFMS_ERROR_CANCELLED=10;			// operation aborted
	FFMS_ERROR_RESAMPLING=11;			// audio resampling (libavresample)

	// Subtypes - what caused the error
	FFMS_ERROR_UNKNOWN = 20;		// unknown error
	FFMS_ERROR_UNSUPPORTED=21;			// format or operation is not supported with this binary
	FFMS_ERROR_FILE_READ=22;			// cannot read from file
	FFMS_ERROR_FILE_WRITE=23;			// cannot write to file
	FFMS_ERROR_NO_FILE=24;				// no such file or directory
	FFMS_ERROR_VERSION=25;				// wrong version
	FFMS_ERROR_ALLOCATION_FAILED=26;	// out of memory
	FFMS_ERROR_INVALID_ARGUMENT=27;	// invalid or nonsensical argument
	FFMS_ERROR_CODEC=28;				// decoder error
	FFMS_ERROR_NOT_AVAILABLE=29;		// requested mode or operation unavailable in this binary
	FFMS_ERROR_FILE_MISMATCH=30;		// provided index does not match the file
	FFMS_ERROR_USER=31;					// problem exists between keyboard and chair

  FFMS_IEH_ABORT = 0;
  FFMS_IEH_CLEAR_TRACK = 1;
  FFMS_IEH_STOP_TRACK = 2;
  FFMS_IEH_IGNORE = 3;

  FFMS_TYPE_UNKNOWN = -1;
  FFMS_TYPE_VIDEO=0;
  FFMS_TYPE_AUDIO=1;
  FFMS_TYPE_DATA=2;
  FFMS_TYPE_SUBTITLE=3;
  FFMS_TYPE_ATTACHMENT=4;

  FFMS_RESIZER_FAST_BILINEAR	= $0001;
	FFMS_RESIZER_BILINEAR		= $0002;
	FFMS_RESIZER_BICUBIC		= $0004;
	FFMS_RESIZER_X				= $0008;
	FFMS_RESIZER_POINT			= $0010;
	FFMS_RESIZER_AREA			= $0020;
	FFMS_RESIZER_BICUBLIN		= $0040;
	FFMS_RESIZER_GAUSS			= $0080;
	FFMS_RESIZER_SINC			= $0100;
	FFMS_RESIZER_LANCZOS		= $0200;
	FFMS_RESIZER_SPLINE			= $0400;

type

  PFFMS_ErrorInfo = ^TFFMS_ErrorInfo;
  TFFMS_ErrorInfo = packed record
    ErrorType, SubType, BufferSize: integer;
    Buffer: PAnsiChar;
  end;

  PFFMS_VideoSource = Pointer;
  PFFMS_AudioSource = Pointer;
  PFFMS_Indexer = Pointer;
  PFFMS_Index = Pointer;
  PFFMS_Track = Pointer;

  PFFMS_Frame=^TFFMS_Frame;
  TFFMS_Frame = packed record
    Data: array [0..3] of PByte;
    Linesize: array[0..3] of integer;
    EncodedWidth, EncodedHeight, EncodedPixelFormat,
    ScaledWidth, ScaledHeight, ConvertedPixelFormat,
    KeyFrame, RepeatPict, InterlacedFrame, TopFieldFirst: integer;
    PictType: ansichar;
    ColorSpace,ColorRange: integer;
    // Introduced in FFMS_VERSION ((2 << 24) | (21 << 16) | (0 << 8) | 0) */
    ColorPrimaries, TransferCharateristics, ChromaLocation: integer;
  end;

  PFFMS_TrackTimeBase = ^TFFMS_TrackTimeBase;
  TFFMS_TrackTimeBase = packed record
  	Num, Den: int64;
  end;

  PFFMS_FrameInfo = ^TFFMS_FrameInfo;
  TFFMS_FrameInfo = packed record
	  PTS: int64;
	  RepeatPict, KeyFrame: integer;
  end;

  PFFMS_VideoProperties = ^TFFMS_VideoProperties;
  TFFMS_VideoProperties = packed record
    FPSDenominator, FPSNumerator, RFFDenominator, RFFNumerator,
    NumFrames, SARNum, SARDen,
    CropTop, CropBottom, CropLeft, CropRight,
    TopFieldFirst: integer;
    ColorSpace, ColorRange: integer;  //FFMS_DEPRECATED
    FirstTime, LastTime: double;
  end;

  PFFMS_AudioProperties = ^TFFMS_AudioProperties;
  TFFMS_AudioProperties = packed record
    SampleFormat, SampleRate, BitsPerSample, Channels: integer;
    ChannelLayout: int64; // should probably be a plain int, none of the constants are >32 bits long
    NumSamples: int64;
    FirstTime, LastTime: double;
  end;
  
{
// Most functions return 0 on success
// Functions without error message output can be assumed to never fail in a graceful way
FFMS_API(void) FFMS_Init(int, int);
FFMS_API(int) FFMS_GetVersion();
FFMS_API(int) FFMS_GetLogLevel();
FFMS_API(void) FFMS_SetLogLevel(int Level);
FFMS_API(FFMS_VideoSource *) FFMS_CreateVideoSource(const char *SourceFile, int Track, FFMS_Index *Index, int Threads, int SeekMode, FFMS_ErrorInfo *ErrorInfo);
FFMS_API(FFMS_AudioSource *) FFMS_CreateAudioSource(const char *SourceFile, int Track, FFMS_Index *Index, int DelayMode, FFMS_ErrorInfo *ErrorInfo);
FFMS_API(void) FFMS_DestroyVideoSource(FFMS_VideoSource *V);
FFMS_API(void) FFMS_DestroyAudioSource(FFMS_AudioSource *A);
FFMS_API(const FFMS_VideoProperties *) FFMS_GetVideoProperties(FFMS_VideoSource *V);
FFMS_API(const FFMS_AudioProperties *) FFMS_GetAudioProperties(FFMS_AudioSource *A);
FFMS_API(const FFMS_Frame *) FFMS_GetFrame(FFMS_VideoSource *V, int n, FFMS_ErrorInfo *ErrorInfo);
FFMS_API(const FFMS_Frame *) FFMS_GetFrameByTime(FFMS_VideoSource *V, double Time, FFMS_ErrorInfo *ErrorInfo);
FFMS_API(int) FFMS_GetAudio(FFMS_AudioSource *A, void *Buf, int64_t Start, int64_t Count, FFMS_ErrorInfo *ErrorInfo);
FFMS_API(int) FFMS_SetOutputFormatV2(FFMS_VideoSource *V, const int *TargetFormats, int Width, int Height, int Resizer, FFMS_ErrorInfo *ErrorInfo); /* Introduced in FFMS_VERSION ((2 << 24) | (15 << 16) | (3 << 8) | 0) */
FFMS_API(void) FFMS_ResetOutputFormatV(FFMS_VideoSource *V);
FFMS_API(int) FFMS_SetInputFormatV(FFMS_VideoSource *V, int ColorSpace, int ColorRange, int Format, FFMS_ErrorInfo *ErrorInfo); /* Introduced in FFMS_VERSION ((2 << 24) | (17 << 16) | (1 << 8) | 0) */
FFMS_API(void) FFMS_ResetInputFormatV(FFMS_VideoSource *V);
FFMS_API(FFMS_ResampleOptions *) FFMS_CreateResampleOptions(FFMS_AudioSource *A); /* Introduced in FFMS_VERSION ((2 << 24) | (15 << 16) | (4 << 8) | 0) */
FFMS_API(int) FFMS_SetOutputFormatA(FFMS_AudioSource *A, const FFMS_ResampleOptions*options, FFMS_ErrorInfo *ErrorInfo); /* Introduced in FFMS_VERSION ((2 << 24) | (15 << 16) | (4 << 8) | 0) */
FFMS_API(void) FFMS_DestroyResampleOptions(FFMS_ResampleOptions *options); /* Introduced in FFMS_VERSION ((2 << 24) | (15 << 16) | (4 << 8) | 0) */
FFMS_API(void) FFMS_DestroyIndex(FFMS_Index *Index);
FFMS_API(int) FFMS_GetSourceType(FFMS_Index *Index);
FFMS_API(int) FFMS_GetSourceTypeI(FFMS_Indexer *Indexer);
FFMS_API(int) FFMS_GetFirstTrackOfType(FFMS_Index *Index, int TrackType, FFMS_ErrorInfo *ErrorInfo);
FFMS_API(int) FFMS_GetFirstIndexedTrackOfType(FFMS_Index *Index, int TrackType, FFMS_ErrorInfo *ErrorInfo);
FFMS_API(int) FFMS_GetNumTracks(FFMS_Index *Index);
FFMS_API(int) FFMS_GetNumTracksI(FFMS_Indexer *Indexer);
FFMS_API(int) FFMS_GetTrackType(FFMS_Track *T);
FFMS_API(int) FFMS_GetTrackTypeI(FFMS_Indexer *Indexer, int Track);
FFMS_API(FFMS_IndexErrorHandling) FFMS_GetErrorHandling(FFMS_Index *Index);
FFMS_API(const char *) FFMS_GetCodecNameI(FFMS_Indexer *Indexer, int Track);
FFMS_API(const char *) FFMS_GetFormatNameI(FFMS_Indexer *Indexer);
FFMS_API(int) FFMS_GetNumFrames(FFMS_Track *T);
FFMS_API(const FFMS_FrameInfo *) FFMS_GetFrameInfo(FFMS_Track *T, int Frame);
FFMS_API(FFMS_Track *) FFMS_GetTrackFromIndex(FFMS_Index *Index, int Track);
FFMS_API(FFMS_Track *) FFMS_GetTrackFromVideo(FFMS_VideoSource *V);
FFMS_API(FFMS_Track *) FFMS_GetTrackFromAudio(FFMS_AudioSource *A);
FFMS_API(const FFMS_TrackTimeBase *) FFMS_GetTimeBase(FFMS_Track *T);
FFMS_API(int) FFMS_WriteTimecodes(FFMS_Track *T, const char *TimecodeFile, FFMS_ErrorInfo *ErrorInfo);
FFMS_DEPRECATED_API(FFMS_Index *) FFMS_MakeIndex(const char *SourceFile, int IndexMask, int DumpMask, TAudioNameCallback ANC, void *ANCPrivate, int ErrorHandling, TIndexCallback IC, void *ICPrivate, FFMS_ErrorInfo *ErrorInfo);
FFMS_API(int) FFMS_DefaultAudioFilename(const char *SourceFile, int Track, const FFMS_AudioProperties *AP, char *FileName, int FNSize, void *Private);
FFMS_API(FFMS_Indexer *) FFMS_CreateIndexer(const char *SourceFile, FFMS_ErrorInfo *ErrorInfo);
FFMS_API(FFMS_Indexer *) FFMS_CreateIndexerWithDemuxer(const char *SourceFile, int Demuxer, FFMS_ErrorInfo *ErrorInfo);
FFMS_DEPRECATED_API(FFMS_Index *) FFMS_DoIndexing(FFMS_Indexer *Indexer, int IndexMask, int DumpMask, TAudioNameCallback ANC, void *ANCPrivate, int ErrorHandling, TIndexCallback IC, void *ICPrivate, FFMS_ErrorInfo *ErrorInfo);
FFMS_API(void) FFMS_TrackIndexSettings(FFMS_Indexer *Indexer, int Track, int Index, int Dump); /* Introduced in FFMS_VERSION ((2 << 24) | (21 << 16) | (0 << 8) | 0) */
FFMS_API(void) FFMS_TrackTypeIndexSettings(FFMS_Indexer *Indexer, int TrackType, int Index, int Dump); /* Introduced in FFMS_VERSION ((2 << 24) | (21 << 16) | (0 << 8) | 0) */
FFMS_API(void) FFMS_SetAudioNameCallback(FFMS_Indexer *Indexer, TAudioNameCallback ANC, void *ANCPrivate); /* Introduced in FFMS_VERSION ((2 << 24) | (21 << 16) | (0 << 8) | 0) */
FFMS_API(void) FFMS_SetProgressCallback(FFMS_Indexer *Indexer, TIndexCallback IC, void *ICPrivate); /* Introduced in FFMS_VERSION ((2 << 24) | (21 << 16) | (0 << 8) | 0) */
FFMS_API(FFMS_Index *) FFMS_DoIndexing2(FFMS_Indexer *Indexer, int ErrorHandling, FFMS_ErrorInfo *ErrorInfo); /* Introduced in FFMS_VERSION ((2 << 24) | (21 << 16) | (0 << 8) | 0) */
FFMS_API(void) FFMS_CancelIndexing(FFMS_Indexer *Indexer);
FFMS_API(FFMS_Index *) FFMS_ReadIndex(const char *IndexFile, FFMS_ErrorInfo *ErrorInfo);
FFMS_API(FFMS_Index *) FFMS_ReadIndexFromBuffer(const uint8_t *Buffer, size_t Size, FFMS_ErrorInfo *ErrorInfo);
FFMS_API(int) FFMS_IndexBelongsToFile(FFMS_Index *Index, const char *SourceFile, FFMS_ErrorInfo *ErrorInfo);
FFMS_API(int) FFMS_WriteIndex(const char *IndexFile, FFMS_Index *Index, FFMS_ErrorInfo *ErrorInfo);
FFMS_API(int) FFMS_WriteIndexToBuffer(uint8_t **BufferPtr, size_t *Size, FFMS_Index *Index, FFMS_ErrorInfo *ErrorInfo);
FFMS_API(void) FFMS_FreeIndexBuffer(uint8_t **BufferPtr);
FFMS_API(int) FFMS_GetPixFmt(const char *Name);
FFMS_API(int) FFMS_GetPresentSources();
FFMS_API(int) FFMS_GetEnabledSources();
}

  procedure FFMS_Init(a:integer; b:integer);stdcall;external 'ffms2.dll' name '_FFMS_Init@8';
  function FFMS_GetVersion:integer;stdcall;external 'ffms2.dll' name '_FFMS_GetVersion@0';
  function FFMS_CreateIndexer(SourceFile:pchar; ErrorInfo:PFFMS_ErrorInfo):PFFMS_Indexer; stdcall; external 'ffms2.dll' name '_FFMS_CreateIndexer@8';
  function FFMS_DoIndexing2(Indexer: PFFMS_Indexer; ErrorHandling:integer; ErrorInfo:PFFMS_ErrorInfo):PFFMS_Index; stdcall; external 'ffms2.dll' name '_FFMS_DoIndexing2@12';
  function FFMS_ReadIndex(const IndexFile: pchar; ErrorInfo: PFFMS_ErrorInfo): PFFMS_Index; stdcall; external 'ffms2.dll' name '_FFMS_ReadIndex@8';
  function FFMS_WriteIndex(const IndexFile: pchar; Index: PFFMS_Index; ErrorInfo: PFFMS_ErrorInfo):integer; stdcall; external 'ffms2.dll' name '_FFMS_WriteIndex@12';
  function FFMS_GetNumTracks(Index: PFFMS_Index): integer; stdcall; external 'ffms2.dll' name '_FFMS_GetNumTracks@4';
  function FFMS_GetNumTracksI(Indexer: PFFMS_Indexer): integer; stdcall; external 'ffms2.dll' name '_FFMS_GetNumTracksI@4';
  function FFMS_GetTrackType(T:PFFMS_Track):integer; stdcall; external 'ffms2.dll' name '_FFMS_GetTrackType@4';
  function FFMS_GetTrackFromIndex(Index:PFFMS_Index; Track:integer):PFFMS_Track; stdcall; external 'ffms2.dll' name '_FFMS_GetTrackFromIndex@8';
  function FFMS_GetTrackFromVideo(V:PFFMS_VideoSource):PFFMS_Track; stdcall; external 'ffms2.dll' name '_FFMS_GetTrackFromVideo@4';
  function FFMS_GetFirstTrackOfType(Index:PFFMS_Index; TrackType:integer; ErrorInfo:PFFMS_ErrorInfo):integer; stdcall; external 'ffms2.dll' name '_FFMS_GetFirstTrackOfType@12';
  function FFMS_GetTrackTypeI(Indexer:PFFMS_Indexer; Track:integer):integer; stdcall; external 'ffms2.dll' name '_FFMS_GetTrackTypeI@8';
  function FFMS_CreateVideoSource(SourceFile:pchar; Track:integer; Index:PFFMS_Index;
              Threads:integer; SeekMode:integer; ErrorInfo:PFFMS_ErrorInfo):PFFMS_VideoSource; stdcall; external 'ffms2.dll' name '_FFMS_CreateVideoSource@24';
  procedure FFMS_DestroyIndex(Index:PFFMS_Index); stdcall; external 'ffms2.dll' name '_FFMS_DestroyIndex@4';
  function FFMS_GetVideoProperties(V: PFFMS_VideoSource):PFFMS_VideoProperties; stdcall; external 'ffms2.dll' name '_FFMS_GetVideoProperties@4';
  function FFMS_GetFrame(V:PFFMS_VideoSource; n:integer; ErrorInfo:PFFMS_ErrorInfo):PFFMS_Frame; stdcall; external 'ffms2.dll' name '_FFMS_GetFrame@12';
  function FFMS_GetFrameInfo(T: PFFMS_Track; Frame: integer): PFFMS_FrameInfo; stdcall; external 'ffms2.dll' name '_FFMS_GetFrameInfo@8';
  function FFMS_GetPixFmt(Name:pchar):integer; stdcall; external 'ffms2.dll' name '_FFMS_GetPixFmt@4';
  function FFMS_GetNumFrames(T:PFFMS_Track):integer; stdcall; external 'ffms2.dll' name '_FFMS_GetNumFrames@4';
  function FFMS_GetTimeBase(T:PFFMS_Track):PFFMS_TrackTimeBase; stdcall; external 'ffms2.dll' name '_FFMS_GetTimeBase@4';
  function FFMS_SetOutputFormatV2(V:PFFMS_VideoSource; TargetFormats: PInteger;
              Width, Height, Resizer:integer; ErrorInfo:PFFMS_ErrorInfo):integer; stdcall; external 'ffms2.dll' name '_FFMS_SetOutputFormatV2@24';
  function FFMS_IndexBelongsToFile(Index: PFFMS_Index; const SourceFile:pchar; ErrorInfo:PFFMS_ErrorInfo):integer; stdcall; external 'ffms2.dll' name '_FFMS_IndexBelongsToFile@12';
  procedure FFMS_DestroyVideoSource(V:PFFMS_VideoSource); stdcall; external 'ffms2.dll' name '_FFMS_DestroyVideoSource@4';

implementation



end.
