unit main;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, ExtCtrls, StdCtrls, ComCtrls,Buttons, Mask, Menus,
  ffmpeg, MMTimer, GifImage, DXRectTracker, DXTrackBar,
  progress, about;

const
  EXENAME = 'DJ''s V2G';

type
  TMainForm = class(TForm)
    dlgOpenFile: TOpenDialog;
    panControl: TPanel;
    btnLeft: TSpeedButton;
    btnRight: TSpeedButton;
    btnPlay: TSpeedButton;
    popMenuTrackbar: TPopupMenu;
    menuSetStart: TMenuItem;
    menuSetEnd: TMenuItem;
    btnOpenFile: TSpeedButton;
    btnSetStart: TSpeedButton;
    btnSetEnd: TSpeedButton;
    txtTime: TMaskEdit;
    btnPause: TSpeedButton;
    btnToGif: TSpeedButton;
    panGif: TPanel;
    Label1: TLabel;
    Label2: TLabel;
    dlgSaveFile: TSaveDialog;
    Label3: TLabel;
    txtGifFPS: TMaskEdit;
    txtGIFWidth: TMaskEdit;
    txtGIFHeight: TMaskEdit;
    panPlayer: TPanel;
    Trackbar: TDXTrackBar;
    Panel1: TPanel;
    imgGif: TImage;
    btnSave: TSpeedButton;
    btnPlayAt: TSpeedButton;
    MainMenu: TPopupMenu;
    MenuAbout: TMenuItem;
    procedure btnOpenFileClick(Sender: TObject);
    procedure btnPlayClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure TrackBarMouseUp(Sender: TObject; Button: TMouseButton;
                              Shift: TShiftState; X, Y: Integer);
    procedure TrackBarKeyUp(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure btnLeftClick(Sender: TObject);
    procedure btnRightClick(Sender: TObject);
    procedure menuSetStartClick(Sender: TObject);
    procedure menuSetEndClick(Sender: TObject);
    procedure btnPauseClick(Sender: TObject);
    procedure btnToGifClick(Sender: TObject);
    procedure btnSaveClick(Sender: TObject);
    procedure btnPlayGifClick(Sender: TObject);
    procedure FormResize(Sender: TObject);
    procedure txtGIFHeightChange(Sender: TObject);
    procedure txtGIFWidthChange(Sender: TObject);
    procedure btnPlayAtClick(Sender: TObject);
    procedure FormContextPopup(Sender: TObject; MousePos: TPoint;
      var Handled: Boolean);
  private
    { Private declarations }
    FCanSetPosition:boolean;
    FSelectionMessage:string;

    procedure CheckGifSize;
    procedure CheckSize;
    procedure ShowSelection;
    procedure OnFileLoaded(Sender:TObject);
    procedure OnPlayerRefreshed(Sender:TObject);
    procedure OnPlayerPaused(Sender:TObject);
    procedure OnPlayerError(Sender:TObject);
    procedure OnPlayerStartIndex(Sender:TObject; ExpectedCount: integer);
    procedure OnPlayerEndIndex(Sender:TObject);
    procedure OnPlayerIndexChanged(Sender:TObject; index: integer);
    procedure OnPlayerSelectionResize(Sender:TObject);
  public
    { Public declarations }
  end;

var
  MainForm: TMainForm;
  Player: TFFMpegPlayer;

implementation

{$R *.dfm}

procedure TMainForm.FormCreate(Sender: TObject);
begin
  Caption:= EXENAME;
  Application.Title:= EXENAME;
  
  FCanSetPosition:=false;
  panPlayer.Left:= 0;
  panPlayer.Top:= 0;
  panPlayer.Width:= ClientWidth;
  panPlayer.Height:= panPlayer.Width * 9 div 16;
  //panPlayer.Color:= clBlack;

  Player:= TFFMpegPlayer.Create(panPlayer);
  Player.OnLoaded:= OnFileLoaded;
  Player.OnRefresh:= OnPlayerRefreshed;
  Player.OnPaused:= OnPlayerPaused;
  Player.OnError:= OnPlayerError;
  Player.OnSelectionResize:= OnPlayerSelectionResize;
  Player.OnStartIndex:= OnPlayerStartIndex;
  Player.OnEndIndex:= OnPlayerEndIndex;
  Player.OnIndexChanged:= OnPlayerIndexChanged;

  TrackBar.TickStyle:= tsManual;
  TrackBar.TickMarks:= tmTopLeft;

  CheckSize;
end;

procedure TMainForm.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  Player.Free;
  imgGif.Picture.Graphic:= nil;
end;

function FormatTime(second:double):string;
var
  h,m,s,ms:integer;
  i:integer;
begin
  i:= round(second);
  h:= i div 3600;
  m:= i mod 3600 div 60;
  s:= i mod 60;
  ms:= round(frac(second)*1000) mod 1000;
  result:= format('%.2d:%.2d:%.2d:%.3d',[h,m,s,ms]);
end;

procedure TMainForm.CheckGifSize;
var
  gWidth, gHeight: integer;
  sel: TSelectedRect;
begin
  sel:= Player.Selection;
  gWidth:= sel.Width;
  gHeight:= sel.Height;

  if (ClientWidth-panGif.Width<gWidth) then
  begin
    gWidth:= ClientWidth - panGif.Width;
    gHeight:= sel.Height * gWidth div sel.Width;
  end;
  if (ClientHeight-panPlayer.Height-Trackbar.Height-panControl.Height<gHeight) then
  begin
    gHeight:= ClientHeight-panPlayer.Height-Trackbar.Height-panControl.Height;
    gWidth:= sel.Width * gHeight div sel.Height;
  end;
  imgGif.Width:= gWidth;
  imgGif.Height:= gHeight;
  imgGif.Left:= (ClientWidth - panGif.Width - imgGif.Width) div 2;
  imgGIf.Top:= (ClientHeight-panPlayer.Height-Trackbar.Height-panControl.Height - imgGif.Height) div 2;
end;

procedure TMainForm.CheckSize;
begin
  ClientHeight:= panPlayer.Height + 210;
  CheckGifSize;
  Player.Paint;
end;

procedure TMainForm.FormResize(Sender: TObject);
begin
  CheckSize;
end;

procedure TMainForm.OnPlayerRefreshed(Sender:TObject);
begin
  FCanSetPosition:= false;
  Trackbar.Position:= Player.FrameIndex;
  txtTime.Text:= FormatTime(Player.Time);
  FCanSetPosition:=true;
end;

procedure TMainForm.OnFileLoaded(Sender:TObject);
begin
  CheckSize;
//  Player.Paint;
  Trackbar.Enabled:=true;
  Trackbar.Max:= Player.FrameCount-1;
  Trackbar.SelStart:=0;
  Trackbar.SelEnd:=0;
  ShowSelection;
  btnPlay.Enabled:=true;
  btnLeft.Enabled:=true;
  btnRight.Enabled:=true;
  btnSetStart.Enabled:=true;
  btnSetEnd.Enabled:=true;
  btnToGif.Enabled:=false;
end;

procedure TMainForm.OnPlayerPaused(Sender:TObject);
begin
  btnPlay.Enabled:= true;
  btnPause.Enabled:= not btnPlay.Enabled;
  btnSetStart.Enabled:=btnPlay.Enabled;
  btnSetEnd.Enabled:=btnPlay.Enabled;
  btnPlayAt.Enabled:= Trackbar.SelEnd > Trackbar.SelStart;
  showSelection;
end;

procedure TMainForm.OnPlayerError(Sender:TObject);
begin
  Showmessage('Error:' + player.ErrorMessage);
end;

procedure TMainForm.OnPlayerSelectionResize(Sender:TObject);
var
  selRect: TSelectedRect;
begin
  selRect:= Player.Selection;
  txtGifWidth.Text:= inttostr(selRect.Width);
  txtGifHeight.Text:= inttostr(selRect.Height);
  CheckGifSize;
end;

procedure TMainForm.OnPlayerStartIndex(Sender:TObject; ExpectedCount: integer);
begin
  DlgProgress.ClientWidth:= Width * 9 div 10;
  DlgProgress.ProgressBar.Min:= 0;
  DlgProgress.ProgressBar.Max:= ExpectedCount;
  self.Enabled:= false;
  DlgProgress.Show;
end;

procedure TMainForm.OnPlayerEndIndex(Sender:TObject);
begin
  DlgProgress.ProgressBar.Max:= 0;
  DlgProgress.Hide;
  self.Enabled:= true;
end;

procedure TMainForm.OnPlayerIndexChanged(Sender:TObject; index:integer);
begin
  if index<DlgProgress.ProgressBar.Max then
  begin
    DlgProgress.ProgressBar.Position:= index;
    Application.ProcessMessages;
  end;
end;

procedure TMainForm.btnOpenFileClick(Sender: TObject);
begin
  if Player.State=statePlaying then
    player.Pause;
  if dlgOpenFile.Execute=true then
  begin
    Caption:='DJ''s V2G - ' + dlgOpenFile.FileName;
    player.FileName:= dlgOpenFile.FileName;
  end;
end;

procedure TMainForm.TrackBarMouseUp(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
var
  pt: TPoint;  
begin
  if (Button=mbLeft) and FCanSetPosition then
  begin
    if (Player.State<>StateNone) and (Player.state<>StateError) then
    begin
      Player.GoFrame(trackbar.Position);
    end;
  end
  else if Button=mbRight then
  begin
    GetCursorPos(pt);
    popMenuTrackbar.Popup(pt.X, pt.Y);
  end;
end;

procedure TMainForm.TrackBarKeyUp(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  case Key of
    VK_LEFT:  player.GoPrevFrame(1);
    VK_RIGHT: player.GoNextFrame(1);
    VK_PRIOR:
      begin
        player.GoPrevFrame(5);
        key:=0;
      end;
    VK_NEXT:
      begin
        player.GoNextFrame(5);
        key:=0;
      end;
  end;
end;

procedure TMainForm.btnLeftClick(Sender: TObject);
begin
  Player.GoPrevFrame(1);
end;

procedure TMainForm.btnRightClick(Sender: TObject);
begin
  Player.GoNextFrame(1);
end;

procedure TMainForm.ShowSelection;
begin
  btnToGif.Enabled:= (Player.State in [StateLoaded, StatePaused])
      and (Trackbar.selEnd>Trackbar.SelStart);
  if Trackbar.selEnd>Trackbar.SelStart then
    FSelectionMessage:=format('%.2fs',[Player.GetIndexTime(Trackbar.SelEnd)-
            Player.GetIndexTime(Trackbar.SelStart)])
  else
    FSelectionMessage:='';
end;

procedure TMainForm.menuSetStartClick(Sender: TObject);
begin
  Trackbar.SelStart:= Trackbar.Position;
  ShowSelection;
  btnPlayAt.Enabled:= Trackbar.SelEnd > Trackbar.SelStart;
end;

procedure TMainForm.menuSetEndClick(Sender: TObject);
begin
  Trackbar.SelEnd:=Trackbar.Position;
  ShowSelection;
  btnPlayAt.Enabled:= Trackbar.SelEnd > Trackbar.SelStart;
end;

procedure TMainForm.btnPlayClick(Sender: TObject);
begin
  if Player.State in [StateLoaded, StatePaused] then
  begin
    btnPlay.Enabled:= false;
    btnPlayAt.Enabled:= btnPlay.Enabled;
    btnPause.Enabled:= not btnPlay.Enabled;
    btnSetStart.Enabled:= not btnPlay.Enabled;
    btnSetEnd.Enabled:= btnSetStart.Enabled;
    btnToGif.Enabled:= btnSetStart.Enabled;
    Player.Play;
  end;
end;

procedure TMainForm.btnPauseClick(Sender: TObject);
begin
  if Player.State in [StatePlaying, StatePlayingAt] then
  begin
    player.Pause;
  end;
end;

procedure TMainForm.btnPlayAtClick(Sender: TObject);
begin
  if Player.State in [StateLoaded, StatePaused] then
  begin
    btnPlay.Enabled:= false;
    btnPlayAt.Enabled:= btnPlay.Enabled;
    btnPause.Enabled:= not btnPlay.Enabled;
    btnSetStart.Enabled:= not btnPlayAt.Enabled;
    btnSetEnd.Enabled:= btnSetStart.Enabled;
    btnToGif.Enabled:= btnSetStart.Enabled;
    Player.PlayAt(Trackbar.SelStart, Trackbar.SelEnd);
  end;
end;

procedure TMainForm.btnToGifClick(Sender: TObject);
var
  width,height,fps:integer;
begin
  width:= strtoint(txtGifWidth.Text);
  height:= strtoint(txtGifHeight.Text);
  fps:= strtoint(txtGifFPS.Text);
  if width>Player.VideoWidth then
  begin
    width:= Player.VideoWidth;
    txtGifWidth.Text:= inttostr(width);
  end;
  if height>Player.VideoHeight then
  begin
    height:= Player.VideoHeight;
    txtGifHeight.Text:= inttostr(height);
  end;
  if fps>player.VideoFPS then
  begin
    fps:= round(Player.VideoFPS);
    txtGifFPS.Text:=inttostr(fps);
  end;
  
  if Player.ToGif(Trackbar.SelStart, Trackbar.SelEnd, width, height, fps) then
  begin
    imgGif.Picture.Assign(Player.GifImage);
    Player.GifImage.Animate:= True;
    btnSave.Enabled:= true;
  end
  else
  begin
    btnSave.Enabled:= false;
  end;
end;

procedure TMainForm.btnPlayGifClick(Sender: TObject);
begin
  Player.GifImage.Animate := True;
end;

procedure TMainForm.btnSaveClick(Sender: TObject);
begin
  if dlgSaveFile.Execute then
  begin
    Player.GifImage.SaveToFile(dlgSaveFile.FileName);
  end;
end;

procedure TMainForm.txtGIFHeightChange(Sender: TObject);
var
  height:integer;
  selRect: TSelectedRect;
begin
  selRect:= Player.Selection;
  height:= strtoint(txtGIFHeight.Text);
  txtGIFWidth.Text:= inttostr(height * selRect.Width div selRect.Height);
end;

procedure TMainForm.txtGIFWidthChange(Sender: TObject);
var
  width: integer;
  selRect: TSelectedRect;
begin
  selRect:= Player.Selection;
  width:= strtoint(txtGIFWidth.Text);
  txtGIFHeight.Text:= inttostr(width * selRect.Height div selRect.Width);
end;

procedure TMainForm.FormContextPopup(Sender: TObject; MousePos: TPoint;
  var Handled: Boolean);
begin
  AboutForm.ShowModal;
end;

end.
